# tiny-json-http

Minimalist `HTTP` client for `GET` and `POST`ing `JSON` payloads

- Zero dependencies: perfect for AWS Lambda or Browserify
- Sensible default: assumes buffered JSON responses
- System symmetry: Node style errback API, or Promises for use with Async/Await

```bash
npm i tiny-json-http --save
```

### api

- `tiny.get(options[, callback])`
- `tiny.post(options[, callback])`
- `tiny.put(options[, callback])`
- `tiny.del(options[, callback)]`

_*callback is optional, tiny methods will return a promise if no callback is provided_

### options

- `url` *required*
- `data` form vars for `tiny.post`, `tiny.put`, and `tiny.delete` otherwise querystring vars for `tiny.get`
- `headers` key/value map used for headers

### callback values

- `err` a real javascript `Error` if there was one
- `data` an object with `headers` and `body` keys

### promises

- if no `callback` is provided to the tiny-json-http methods, a promise is returned
- perfect for use of async/await

## example

#### With Async / Await

```javascript
var tiny = require('tiny-json-http')
var url = 'http://www.randomkittengenerator.com'

(async () => {
  try {
    const result = await tiny.get({url})
    console.log(result)
  } catch (err) {
    console.log('ruh roh!', err)
  }
})
```

#### With Callback

```javascript
var tiny = require('tiny-json-http')
var url = 'http://www.randomkittengenerator.com'

tiny.get({url}, function __got(err, result) {
  if (err) {
    console.log('ruh roh!', err)
  }
  else {
    console.log(result)
  }
})
```

Check out the tests for more examples! :heart_decoration:
